(*******************************************************************************
**
**  RegisterExtension - I played with John Lam's code for a couple of days
**                      using D3.02 without ever getting it to work.  After much
**                      head scratching, general pondering, VCL source
**                      inspection and scanning of the MSCDN, I discovered a
**                      minor problem in the ComServer registration code.  The
**                      default code checks for a space in the DLL path (a quick
**                      check for a long file name) and surrounds the path in
**                      quotes, before writing to the registry.  Also the base
**                      class does no 'threading' model registering in the
**                      registry.  I eventually tripped over the solution after
**                      reading an article in Delphi Magazine by Steve Teixeira
**                       - thanks Steve.
**                      Some of this code is based on what I read.
**
**                      NOTE: The extra registration code for correcting the
**                            ActiveX server registration, is nott needed under
**                            Delphi 4, as is has been corrected in the factory
**                            library code.
**
**                      To see the code in action, compile this project under D3
**                      (I don't know if it works on any other version)
**                      select ('Run',  'Register ActiveX Server') then run an
**                      Explorer window.  Open 'My Computer', and at the bottom
**                      should be the Delphi 3 Shell Extension - enjoy!
**
**  Useful Tip:         The MSCDN states that there is a way to debug Shell
**                      extension DLL/OCX's if you use a 'Just in time' debugger.
**                      D3 and D4 supports this, so this is how it works:
**                      1)  Generate your Shell Extension in Delphi as a DLL.
**                      2)  Set the 'Run' parameters 'Host Application' to
**                            C:\WINDOWS\EXPLORER.EXE.
**                      3)  Click on the Win95/98/NT 'Start' button and select
**                            'Shutdown'.
**                      4)  Hold 'Ctrl', 'Shift' and 'Alt' and click on 'No'
**                            with the mouse.  This shuts the shell down leaving
**                            any applications still running (including Delphi!)
**                      5)  Press F9 (or 'Run' and 'Run' from the Delphi menu)
**                          and hey presto, the shell appears.
**                      6)  Bring up an instance of Explorer (WIN-E keys), and
**                          you can set breakpoints and debug your extension
**                          like any other DLL!
**
**                      To get the shell back, just Dbl-Click on the desktop
**                      (this brings up the Task Manager), select 'File' then
**                      'Run Application' and select 'C:\WINDOWS\EXPLORER.EXE'
**
**                      This works because of a little known fact about Explorer.
**                      The first running occurance of Explorer IS the Shell.
**                      All you are effectively doing, is to run the first
**                      occurance of Explorer under Delphi's debugger!
**
**  Author: Andy Strong - andrews@nbaqsl.co.uk    11 January 1999
**
*******************************************************************************)
unit RegisterExtension;

interface

uses
  ComObj;


type
  TNamespaceExtensionFactory = class(TComObjectFactory)
  protected
    function GetProgID: string; override;
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

implementation

uses
  Windows, SysUtils, ShellView, ShlObj;

(*
**  Returns a Win95 long path as a Win3.1 shortpath, or the passed in path
**  if not needed to be converted.
*)
function GetShortPath(LongPath: ansiString): ansistring;
var
  szShortPath,
  szLongPath: array[0..MAX_PATH] of char;
  PLen: Longint;
begin
  Result := LongPath;                       { Default - return existing string }
  StrPCopy(szLongPath, LongPath);           { Copy Long path in to work area }
  PLen := GetShortPathName(szLongPath, szShortPath, MAX_PATH);
  if not ((PLen = 0) or (Plen > MAX_PATH)) then{ no error }
    Result := StrPas(szShortPath);          { Use shortened path }
end;

procedure CreateRegKeyEx(const Key, ValueName: string; Value: PChar;
                          Kind, Size: DWORD; RootKey: HKEY);
var
  Handle: HKey;
  Status,
  Disposition: Integer;
begin
  Status := RegCreateKeyEx(RootKey, PChar(Key), 0, '',
    REG_OPTION_NON_VOLATILE, KEY_READ or KEY_WRITE or KEY_SET_VALUE, nil,
      Handle, @Disposition);
  if Status = 0 then
  begin
    Status := RegSetValueEx(Handle, PChar(ValueName), 0, Kind, Value, Size);
    RegCloseKey(Handle);
  end;
  if Status <> 0 then
    raise EWin32Error.Create(SysErrorMessage(Status));
end;

procedure DeleteRegValue(const Key, ValueName: string; RootKey: HKEY);
var
  Handle: HKEY;
  Status: Integer;
begin
  Status := RegOpenKey(RootKey, PChar(Key), Handle);
  if Status = 0 then
  begin
    Status := RegDeleteValue(Handle, PChar(ValueName));
    RegCloseKey(Handle);
  end;
  if Status <> 0 then
    raise EWin32Error.Create(SysErrorMessage(Status));
end;


{ TNamespaceExtensionFactory }

function TNamespaceExtensionFactory.GetProgID: string;
begin
  Result := '';                             { No need for namespaces apparently }
end;

procedure TNamespaceExtensionFactory.UpdateRegistry(Register: Boolean);
const
  //  Root the extension under the 'My Computer' icon.
  NamespaceKey = 'SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\MyComputer\Namespace\';
  //  Needed to allow extension to run under NT.
  ApproveKey = 'SOFTWARE\Microsoft\Windows\CurrentVersion\Shell Extensions\Approved\';
var
  Temp,
  ClsID: string;
  Value: DWORD;
begin
  ClsID := GUIDToString(ClassID);
  inherited UpdateRegistry(Register);
  if Register then
  begin
    Temp := GetShortPath(ComServer.ServerFileName);
    //  Overwrite the base class CLSID value with the proper entry!
    CreateRegKey('CLSID\' + ClsID + '\' + ComServer.ServerKey, '', Temp);

    // Shell extension must be approved under NT.
    if Win32Platform = VER_PLATFORM_WIN32_NT then
      CreateRegKeyEx(ApproveKey, ClsId, PChar(Description), REG_SZ,
        Length(Description) + 1, HKEY_LOCAL_MACHINE);

    // Write away threading model
    CreateRegKeyEx('CLSID\' + ClsId + '\InProcServer32\', 'ThreadingModel',
      'Apartment'#0, REG_SZ, 10, HKEY_CLASSES_ROOT);

    // Junction for this namespace extension is "My Computer"
    CreateRegKeyEx(NameSpaceKey + ClsId, '', PChar(Description), REG_SZ,
      Length(Description) + 1, HKEY_LOCAL_MACHINE);
    Value := SFGAO_FOLDER;// or SFGAO_HASSUBFOLDER;
    CreateRegKeyEx('CLSID\' + ClsId + '\ShellFolder\', 'Attributes',
      @Value, REG_BINARY, SizeOf(DWORD), HKEY_CLASSES_ROOT);
    // Default icon is contained within this file
    CreateRegKey('CLSID\' + ClsId + '\DefaultIcon', '', Temp + ',0');
  end
  else begin
    // Remove junction
    RegDeleteKey(HKEY_LOCAL_MACHINE, PChar(NameSpaceKey + ClsId));
    // Remove approval on NT
    if Win32Platform = VER_PLATFORM_WIN32_NT then
      DeleteRegValue(ApproveKey, ClsId, HKEY_LOCAL_MACHINE);
  end;
end;

end.
