//
// Delphi 3.0 Implementation of IShellFolder Interface
// (C)1997 Naleco Research Inc.
// Written by Dr. John F. Lam
//
(*******************************************************************************
**
**  This ShellView object used to be descended from the TComObject object.  This
**  required the object to have a CLSID, be registered in the Registry,
**  and required a COM factory to create it.  Descending the View object from
**  TInterfacedObject, negates the requirement for a Factory, or giving it a
**  CLSID.  Since we only want to create this object from within our extension
**  (NOT a free standing version outside of explorer) it doesn't need to have
**  its own CLSID and factory.  Therefore, we can treat it as a normal Delphi
**  object, and create it accordingly. My thanks to Joel Milne for clearing up
**  my understanding of the difference between the two Delphi COM base classes.
**
**   - ajs 14Jan99
**
**  Problems: There are only two outstanding problems I know of;
**            1)  If there is more than one of the View objects on the screen,
**                ONLY the first one created responds to the Right-Click and
**                displays the 'File Open' dialogue.
**            2)  We need to respond to the IShellView.SaveViewState call and
**                save the current view options, otherwise Explorer loses the
**                'View' settings.
**
**  If anyone can fix these, please drop me an e-mail.
**
**  Dedication:
**  The work, time, blood, sweat, (virtual) tears and swearing that went into
**  making this example code function more or less correctly, is hereby
**  dedicated to the Delphi community.  Hopefully to avoid anyone else suffering
**  undue stress to gain a working framework of an Explorer Namespace extension.
**  In the spirit of this, anyone who uses this code, and adds further working
**  Interfaces (eg. Menu manipulation, Icon handling, Drag-Drop handling etc),
**  I would be grateful for some stress relief myself<BG>.  All such examples
**  gratefully received.  Lets make Explorer extensions easier for Delphi
**  programmers.
**
*******************************************************************************)
unit ShellFolder;

interface

uses Windows, ActiveX, ComObj, ComServ, ShlObj, ShellView;

const
  CLSID_RADFindBrowser: TGUID = '{23CE4E06-73A7-11D0-BC62-00A0243ABE0B}';

type
  TShellFolderImpl = class( TComObject, IShellFolder, IPersistFolder )
  protected
    // We do the following declaration to Alias the Initialize function
    // that is normally implemented in IPersistFolder to the correct
    // function implemented in TShellFolderImpl
    function IPersistFolder.Initialize = IPersistFolder_Initialize;
  public
    // IShellFolder
    function ParseDisplayName(hwndOwner: HWND;
      pbcReserved: Pointer; lpszDisplayName: POLESTR; out pchEaten: ULONG;
      out ppidl: PItemIDList; var dwAttributes: ULONG): HResult; stdcall;
    function EnumObjects(hwndOwner: HWND; grfFlags: DWORD;
      out EnumIDList: IEnumIDList): HResult; stdcall;
    function BindToObject(pidl: PItemIDList; pbcReserved: Pointer;
      const riid: TIID; out ppvOut: Pointer): HResult; stdcall;
    function BindToStorage(pidl: PItemIDList; pbcReserved: Pointer;
      const riid: TIID; out ppvObj: Pointer): HResult; stdcall;
    function CompareIDs(lParam: LPARAM;
      pidl1, pidl2: PItemIDList): HResult; stdcall;
    function CreateViewObject(hwndOwner: HWND; const riid: TIID;
      out ppvOut: Pointer): HResult; stdcall;
    function GetAttributesOf(cidl: UINT; var apidl: PItemIDList;
      var rgfInOut: UINT): HResult; stdcall;
    function GetUIObjectOf(hwndOwner: HWND; cidl: UINT; var apidl: PItemIDList;
      const riid: TIID; prgfInOut: Pointer; out ppvOut: Pointer): HResult; stdcall;
    function GetDisplayNameOf(pidl: PItemIDList; uFlags: DWORD;
      var lpName: TStrRet): HResult; stdcall;
    function SetNameOf(hwndOwner: HWND; pidl: PItemIDList; lpszName: POLEStr;
      uFlags: DWORD; var ppidlOut: PItemIDList): HResult; stdcall;

    // IPersist
    function GetClassID(out classID: TCLSID): HResult; stdcall;

    // IPersistFolder
    function IPersistFolder_Initialize(pidl: PItemIDList): HResult; virtual; stdcall;

  end;

implementation

uses
  RegisterExtension;

function TShellFolderImpl.ParseDisplayName(hwndOwner: HWND;
  pbcReserved: Pointer; lpszDisplayName: POLESTR; out pchEaten: ULONG;
  out ppidl: PItemIDList; var dwAttributes: ULONG): HResult;
begin
  MessageBox( 0, 'TShellFolderImpl.ParseDisplayName', nil, 0 );
  Result := E_NOTIMPL;
end;

function TShellFolderImpl.EnumObjects(hwndOwner: HWND; grfFlags: DWORD;
  out EnumIDList: IEnumIDList): HResult;
begin
  MessageBox( 0, 'TShellFolderImpl.EnumObjects', nil, 0 );
  Result := E_NOTIMPL;
end;

function TShellFolderImpl.BindToObject(pidl: PItemIDList; pbcReserved: Pointer;
  const riid: TIID; out ppvOut: Pointer): HResult;
begin
  MessageBox( 0, 'TShellFolderImpl.BindToObject', nil, 0 );
  Result := E_NOTIMPL;
end;

function TShellFolderImpl.BindToStorage(pidl: PItemIDList; pbcReserved: Pointer;
  const riid: TIID; out ppvObj: Pointer): HResult;
begin
  MessageBox( 0, 'TShellFolderImpl.BindToStorage', nil, 0 );
  Result := E_NOTIMPL;
end;

function TShellFolderImpl.CompareIDs(lParam: LPARAM;
  pidl1, pidl2: PItemIDList): HResult;
begin
  MessageBox( 0, 'TShellFolderImpl.CompareIDs', nil, 0 );
  Result := E_NOTIMPL;
end;

// This function is called immediately after IPersistFolder::Initialize is
// called by the browser. Here, we construct the new window for our view
// object
function TShellFolderImpl.CreateViewObject(hwndOwner: HWND; const riid: TIID;
  out ppvOut: Pointer): HResult;
var
  shellView: IShellView;
begin
  try
    if IsEqualGUID( riid, IShellView ) then
    begin
      ShellView := TShellViewImpl.Create;
      //  The QueryInterface call increments its reference count via TObjects
      //  QueryInterface call during the inherited QueryInterface chain
      //   - ajs 11Jan99
      Result := (ShellView as IUnknown).QueryInterface(riid, ppvOut)
    end else
      Result := E_NOINTERFACE;
  except
    on E: EOleSysError do Result := E.ErrorCode;
  else
    Result := E_UNEXPECTED;
  end;
end;

function TShellFolderImpl.GetAttributesOf(cidl: UINT; var apidl: PItemIDList;
  var rgfInOut: UINT): HResult;
begin
  MessageBox( 0, 'TShellFolderImpl.GetAttributesOf', nil, 0 );
  Result := E_NOTIMPL;
end;

function TShellFolderImpl.GetUIObjectOf(hwndOwner: HWND; cidl: UINT; var apidl: PItemIDList;
  const riid: TIID; prgfInOut: Pointer; out ppvOut: Pointer): HResult;
begin
  MessageBox( 0, 'TShellFolderImpl.GetUIObjectOf', nil, 0 );
  Result := E_NOTIMPL;
end;

function TShellFolderImpl.GetDisplayNameOf(pidl: PItemIDList; uFlags: DWORD;
  var lpName: TStrRet): HResult;
begin
  MessageBox( 0, 'TShellFolderImpl.GetDisplayNameOf', nil, 0 );
  Result := E_NOTIMPL;
end;

function TShellFolderImpl.SetNameOf(hwndOwner: HWND; pidl: PItemIDList; lpszName: POLEStr;
  uFlags: DWORD; var ppidlOut: PItemIDList): HResult;
begin
  MessageBox( 0, 'TShellFolderImpl.SetNameOf', nil, 0 );
  Result := E_NOTIMPL;
end;

{ IPersistFolder }

function TShellFolderImpl.GetClassID(out classID: TCLSID): HResult;
begin
  classID := CLSID_RADFindBrowser;

  MessageBox( 0, 'TShellFolderImpl.GetClassID', nil, 0 );
  Result := NOERROR;
end;

// This function is called from the Shell's implementation of
// IShellFolder::BindToObject. It passes a PIDL to our object so that we can
// construct fully qualified PIDL's to each of our contained objects if
// necessary. In our example, this is NOT necessary, since our folders are
// simply views onto the existing shell namespace.
function TShellFolderImpl.IPersistFolder_Initialize(pidl: PItemIDList): HResult;
begin
  Result := NOERROR;
end;

initialization
//  Old Registration Code - ajs 18 Dec 98
//  TComObjectFactory.Create( ComServer, TShellFolderImpl, CLSID_RADFindBrowser,
//    '', 'Delphi 3.0 RADFind Explorer Extension', ciMultiInstance );

//  Registration code to correctly register under Delphi 3 (works on D4 as well)
//  - ajs 20 Dec 98
  TNamespaceExtensionFactory.Create(ComServer, TShellFolderImpl, CLSID_RADFindBrowser,
    '', 'Delphi 3.0 RADFind Explorer Extension', ciMultiInstance)


end.
