(*******************************************************************************
**
**  VIEWFORM  - Right-hand pane for the Explorer name space extension.
**              Original code written by John Lam.  Modified (and got working!)
**              by Andy Strong; some of this is based on suggestions
**              by Joel Milne (thanks for saving me lots of time Joel!)
**
**            - ajs 14Jan99
**
**              Contact me (Andy Strong:  andrews@nbaqsl.co.uk)
**                or       (Joel Milne:   jmilne@softmosis.ca)
**
*******************************************************************************)
unit ViewForm;

interface

uses
  Windows, Forms, ShlObj, Dialogs, StdCtrls, ComCtrls, ExtCtrls, Menus,
  Classes, Controls, Messages;

type
  TView = class(TForm)
    PopupMenu1: TPopupMenu;
    N1: TMenuItem;
    Panel1: TPanel;
    OpenDialog1: TOpenDialog;
    RichEdit1: TRichEdit;
    procedure N1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormShow(Sender: TObject);

  private
    { Private declarations }
    FSHBrowser: IShellBrowser;
    FShellView: IShellView;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    constructor CreateSHView(AOwner: TComponent; SHBrowser: IShellBrowser; SHView: IShellView);
    destructor Destroy; override;

  end;

var
  View: TView;

implementation

{$R *.DFM}

uses
  SysUtils;


(*
**  Stop the default constructor being called for this type of form.
*)
constructor TView.Create(AOwner: TComponent);
begin
  ShowMessage('Use CreateSHView for this View COM object');
  Abort;                                    // Silent Exception
end;

(*
**  Special Constructor.  Allows the passing of the ShellBrowser and ShellView
**  object in for use when we gain focus.
**  For this form to behave correctly under Explorer's control, it MUST be
**  a child window.  The original code did not have this Windows style set.
*)
constructor TView.CreateSHView(AOwner: TComponent; SHBrowser: IShellBrowser; SHView: IShellView);
begin
  FSHBrowser := nil;
  FShellView := nil;                        // In case Inherited constructor fails
  inherited Create(AOwner);

  FSHBrowser := SHBrowser;
  FShellView := SHView;
  FSHBrowser._AddRef;                       // Ensure it stays alive until we die
  FShellView._AddRef;                       // Ensure it stays alive until we die

  SetWindowLong(Handle, GWL_STYLE, WS_CHILD);//  MUST HAVE CHILD STYLE ONLY.
end;

(*
**  Reduce active counts on our references for the control objects.
*)
destructor TView.Destroy;
begin
  try
    if Assigned(FSHBrowser) then
      FSHBrowser._Release;                  // Decrement reference count
    if Assigned(FShellView) then
      FShellView._Release;                  // Decrement reference count
  finally
    inherited Destroy;
  end
end;

(*
**  Open 'Open' Dialogue to load a diferent file.
*)
procedure TView.N1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
    RichEdit1.Lines.LoadFromFile( OpenDialog1.FileName );
end;

(*
**  In a single documented place in the MSCDN, there is this little reference to
**  the OnViewWindowActive callback to the Shell.  This must be called when the
**  our form gains focus, to let the shell subsequently call the UIActivate
**  interface callback.
**  Set the focus to the rich edit control.
*)
procedure TView.FormActivate(Sender: TObject);
begin
//  RichEdit1.Lines.LoadFromFile( 'c:\Delphi 3 and COM.rtf' );
  FSHBrowser.OnViewWindowActive(FShellView);
  RichEdit1.SetFocus;                       // Force focus if requested - ajs
end;

(*
**  Load our demo text.
*)
procedure TView.FormShow(Sender: TObject);
begin
  if FileExists('c:\BootLog.txt') then
    RichEdit1.Lines.LoadFromFile('c:\BootLog.txt')
  else
  if FileExists('c:\config.sys') then
    RichEdit1.Lines.LoadFromFile('c:\config.sys')
  else
    RichEdit1.Lines.Add('BootLog.txt or Config.sys not found')
end;

end.
